function v = centre_ellipse_moid(v0,idv,I)
% compute centre of the ellipse from its intensity image I
% method --  inside-outside difference
% only parameters of ellipse indexed by idv are optimized, input v0, idv, output v
% the optimization algorithm chosen to be simplex!
% input:
%   v0 -- initial paramters of ellipse , 5x1 vector
%   idv -- index of the paramters to be optimized
%   I -- intensity image
% output:
%   v -- optimized paramters of the ellipse


 % optimize ellipse

% v0 = [ra/2,rb/2,ang,c(1),c(2)]';
 e0 = norm(ellipse_fit(v0,double(I)));
 
 % simplex optimization
 x0 = v0(idv);
 x = fminsearch(@(x) ellipse_fit1(x,idv,v0,double(I)),x0);
 v = v0;
 v(idv) =x;
e1 = norm(ellipse_fit(v,double(I)));

% h=figure();
% imshow(I);
% image(I)
% ellipse(a1,b1,ang1,cx1,cy1,'r');
% ellipse(v0(1),v0(2),v0(3),v0(4),v0(5),'b');
%   dcx = v(4)-v0(4)
%   dcy = v(5)-v0(5)
% close(h);

%  dang = ang1-ang
 
end

function err = ellipse_fit1(x,idv,v,I)
v(idv)=x;
err = ellipse_fit(v,I);
end

function err = ellipse_fit(v,I)
a = v(1);
b = v(2);
ang = v(3);
cx = v(4);
cy = v(5);
[H,W] = size(I);

co=cos(ang);
si=sin(ang);
the=linspace(0,2*pi,360);
x = a*cos(the)*co-si*b*sin(the)+cx;
y = a*cos(the)*si+co*b*sin(the)+cy; 
n = length(x);

R = [co,-si;si,co];
txy = R*[-a*sin(the')';b*cos(the')']; % tangent

a_txy = sqrt(txy(1,:).^2+txy(2,:).^2);
tt = [txy(1,:)./a_txy;txy(2,:)./a_txy];
xy = [x-cx;y-cy]; 
p_xy = dot(xy,tt,1);
nl = xy-[p_xy.*tt(1,:);p_xy.*tt(2,:)];
a_nl = sqrt(nl(1,:).^2+nl(2,:).^2);
nl(1,:) = nl(1,:)./a_nl;
nl(2,:) = nl(2,:)./a_nl;

jj = [-3:0.1:3]'; % grid width 3, interval 0.1
len_jj = length(jj);
nl_1 = nl(ones(len_jj,1),:).*jj(:,ones(1,n));
nl_2 = nl(2*ones(len_jj,1),:).*jj(:,ones(1,n));
x_grid = x(ones(len_jj,1),:);
xx = x_grid + nl_1;
y_grid = y(ones(len_jj,1),:);
yy = y_grid + nl_2;

i = ceil(xx); j = ceil(yy);
i_1 = i-1; j_1 = j-1;
ix = xx-i+1; jy = yy-j+1;
b1 = I( j_1(:)+(i_1(:)-1)*H   );
b2 = I(j_1(:)+(i(:)-1)*H) - I(j_1(:)+(i_1(:)-1)*H);
b3 = I(j(:)+(i_1(:)-1)*H) - I(j_1(:)+(i_1(:)-1)*H);
b4 = I(j_1(:)+(i_1(:)-1)*H) - I(j_1(:)+(i(:)-1)*H) ...
    - I(j(:)+(i_1(:)-1)*H) + I(j(:)+(i(:)-1)*H);

v = b1 + b2.*ix(:) + b3.*jy(:) +b4.*ix(:).*jy(:) ;

v = reshape(v,len_jj,n);
e = mean(v(ceil(len_jj/2)+1:end,:),1) - mean(v(1:floor(len_jj/2),:),1);

err = e.^2;
% err = 1/sum(e);
err = 1/sum(err);

end